function patchSim = LGmodel(Sp, wp, I)
% This function computes a patch simulated for the Locally Gaussian model
% learnt on the N nearest neighbours from the set Sp. The patch is of size
% wp X wp. In Sp we find the N x and y coordinates of the nearest
% neighbours. 

N = size(Sp,1);% The set's size

row = Sp(:,1);% the x coordinate of the nearest neighbours
col = Sp(:,2);% the y coordinate of the nearest neighbours

% Matriw P is composed of the patches of Sp in their vectorized version
P = zeros(N,wp*wp*size(I,3));
for k=1:N
    patch = I(row(k):row(k)+wp-1,col(k):col(k)+wp-1,:);
    P(k,:) = patch(:);
end

% mean of the gaussian vector we use to model the patch
MU = mean(P,1);

%%%
%%% simulate patch 
%%%

P = P - ones(N,1)*MU;
P = P/sqrt(N);

[V,D] = eig((P)*(P).');% V contains the eigen vectors of P*P' in each row
D = sqrtm(D);% en D estan los valores propios de PP', que son los lamba^2
D = real(D);
U = P.'*V;
for i=1:N
    normU = norm(U(:,i));
    U(:,i) = U(:,i)/max(normU,eps);
end

X = randn(1,N);
Y = X*D*U.' + MU;

patchSim = reshape(Y,[wp wp size(I,3)]);

end