% function synthesis_MSLG
% given the input u0, synthesis_LG returns a synthesis of u0 in w using
% multiscale locally gaussian models (LG)
% p0 is the position of the seed patch
% r the output/input ratio
% n the patch size
% m number of nearest neighbours
% K number of scales used
% o overlap proportion
function w = synthesis_MSLG(u0,p0,r,n,m,K,o)

alpha = 2;
eta = 0.5;

[M,N,C] = size(u0); % input size
wo = round(o * n); % overlap size
% seed patch position
x0 = p0(1);
y0 = p0(2);

u = [];% sample pyramid
z = [];% low resolution sample pyramid (z=u*Gauss)
u{1,1} = u0;% initial level of the pyramid is the input sample

% compute pyramid decomposition of u
for i=1:K-1
    [u{1,i+1},z{1,i}] = zoom_out_pyr(u{1,i},alpha);
end
if isempty(i)
    i=0;
end
z{1,i+1} = zeros(size(u{1,i+1}));

% initialize v the low resolution of the synthesis
% result at each scale. v is initialized at the
% coarsest scale
v = initializeV(n,wo,M,N,C,alpha,K,r);

% synthesis for scales k=K-1 --> k=1
for k=K-1:-1:1
    disp(['Processing k= ', num2str(k), '...']);
    w = synthLG_HF(u{1,k+1},v,z{1,k+1},n,wo,m,k,K,r,eta,x0,y0);
    v = zoom_in(w,alpha);
end

% synthesis for scales k=0
disp(['Processing k= ', num2str(0), '...']);
if K>1
    w = synthLG_HF(u0,v,z{1,1},n,wo,m,k-1,K,r,eta,[],[]);
else
    w = synthLG_HF(u0,v,z{1,1},n,wo,m,K-1,K,r,eta,x0,y0); 
end
